Module buf

Module buf 

Strongly-typed buffers

This module is mostly internal to the crate, but is public because its types appear as return values and arguments.

Structs

GenericFlexBuffer
Handle around a growable GPU buffer
MappedBufferTag
Helper struct to make a mapped version of a storage buffer
MappedImage
Handle to a mapped ImageReadBuffer, which unmaps the image when dropped

Enums

BufferSizeError
Error type when resizing a buffer beyond its limit
BufferType
Buffer type for error reporting

Traits

BufferItemCount
Trait for types which have a certain number of items
BufferTag
Tag associated with a particular GenericFlexBuffer

Type Aliases

ArrayBuffer
Resizable array buffer
ImageBuffer
Resizable image buffer
ImageReadBuffer
Buffer for reading data back from the GPU