BufferSizeError

Enum BufferSizeError 

pub enum BufferSizeError {
    TooLarge {
        requested_size: u64,
        buffer_type: BufferType,
    },
}
Error type when resizing a buffer beyond its limit

We check against maximum buffer sizes (from the WebGPU spec) and return an error immediately, instead of deferring the error to the point where the buffer is used.

Variants



TooLarge

Buffer size is too large for the requested buffer usage

Fields

requested_size: u64

Size requested (in bytes)

buffer_type: BufferType

Buffer type (which determines the max size)