BufferTag

Trait BufferTag 

pub trait BufferTag {
    type T;

    // Required method
    fn usage() -> u32;
}
Tag associated with a particular GenericFlexBuffer

The tag type serves two purposes:

  • It declares the storage type and usage bits for the buffer
  • It makes buffers strongly typed, so that two buffers with equivalent storage and usage bits can be distinct types.

Required Associated Types

type T

Data type stored in the buffer

Required Methods

fn usage() -> u32

Usage bits for the buffer

This must be a union of wgpu::BufferUsages values

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors