Gpu

Struct Gpu 

pub struct Gpu {
    pub device: Device,
    pub queue: Queue,
}
Handle to a GPU device

Fields

device: Device

GPU device

queue: Queue

GPU queue

Implementations

Source

impl Gpu

pub async fn init() -> Result<Gpu, InitError>

Returns a Gpu object with customized settings

Non-default settings are as follows:

This is a helper function for simplicity; more sophisticated systems will likely construct the adapter, device, and queue themselves.

pub fn read_buffer_for<T: BufferTag>( &self, buf: &ImageBuffer<T>, ) -> ImageReadBuffer<T>

Returns a readable buffer for the given image buffer

pub fn map<'a, T: BufferTag>( &self, buf: &'a mut ImageReadBuffer<T>, ) -> MappedImage<'a, T>

Maps a readable image buffer, returning a mapped image

pub fn read_vec<T: FromBytes + Immutable + Clone + Copy>( &self, buf: &Buffer, ) -> Vec<T>

Debug function to read from a buffer to a Vec<T>