pub struct Buffers { /* private fields */ }Expand description
Buffers for rendering, which control the rendered image size
This object is constructed by Context::buffers and may only be used with
that particular Context.
A successfully constructed Buffers object also guarantees infallible
construction of an ImageReadBuffer object of the same size.
Implementations
Sourceimpl Buffers
impl Buffers
Sourcepub fn image_size(&self) -> VoxelSize
pub fn image_size(&self) -> VoxelSize
Returns the current image size
Sourcepub fn image_storage_buffer(&mut self) -> &ImageBuffer<GeomBufferTag>
pub fn image_storage_buffer(&mut self) -> &ImageBuffer<GeomBufferTag>
Returns a handle to the image storage buffer
This is intended for subsequent shaders which want to use the
GeometryPixel image data without copying to the CPU. It requires a
exclusive borrow of the Buffers object (and then extends that
lifetime) so that other callers can’t simultaneously touch the buffer.