Context

Struct Context 

pub struct Context { /* private fields */ }
Context for 3D (combined heightmap and normal) rendering

Implementations

Source

impl Context

pub fn new(gpu: &Gpu) -> Self

Build a new 3D rendering context, given a device and queue

If render timestamps are desirable, then the device should be initialized with wgpu::Features::TIMESTAMP_QUERY.

pub fn buffers(&self, image_size: VoxelSize) -> Result<Buffers, BuffersError>

Builds a new Buffers object for the given render size

An image rendered with the resulting buffers will have the given width and height; image_size.depth() sets the number of voxels to evaluate within each pixel of the image (stacked into a column going into the screen).

pub fn image_buffer(&self, buffers: &Buffers) -> ImageReadBuffer

Returns an ImageReadBuffer, sized to read from a Buffers object

This is infallible because the Buffers constructor also ensures that the image size is appropriate for the image read buffer (even though it’s constructed separately).

pub fn shape(&self, shape: &VmShape) -> Result<RenderShape, RenderShapeError>

Builds a new RenderShape object for the given shape

pub fn run( &self, shape: &RenderShape, buffers: &Buffers, out: &mut ImageReadBuffer, settings: RenderConfig, ) -> Result<Image, MissingVar>

Renders the image, with a blocking wait to read pixel data from the GPU

This function is not present when built for the wasm32 target

pub fn run_with_vars( &self, shape: &RenderShape, vars: &ShapeVars<f32>, buffers: &Buffers, out: &mut ImageReadBuffer, settings: RenderConfig, ) -> Result<Image, MissingVar>

Renders the image, with a blocking wait to read pixel data from the GPU

This function is not present when built for the wasm32 target

pub async fn run_async( &self, shape: &RenderShape, buffers: &Buffers, out: &mut ImageReadBuffer, settings: RenderConfig, ) -> Result<Image, MissingVar>

Renders the image, with a blocking wait to read pixel data from the GPU

This function is only relevant for the web target

pub async fn run_with_vars_async( &self, shape: &RenderShape, vars: &ShapeVars<f32>, buffers: &Buffers, out: &mut ImageReadBuffer, settings: RenderConfig, ) -> Result<Image, MissingVar>

Renders the image, with a blocking wait to read pixel data from the GPU

This function is only relevant for the web target

pub fn submit( &self, shape: &RenderShape, buffers: &mut Buffers, out: Option<&mut ImageReadBuffer>, settings: &RenderConfig, ) -> Result<(), MissingVar>

Submits a single image to be rendered on the GPU

The resulting image (as a buffer of GeometryPixel data) is available on the GPU in buffers.image_storage_buffer().

If out is present, then the rendered image is also copied to that ImageReadBuffer (which may then be mapped for CPU reading by map_image or map_image_async).

pub fn submit_with_vars( &self, shape: &RenderShape, vars: &ShapeVars<f32>, buffers: &Buffers, out: Option<&mut ImageReadBuffer>, settings: &RenderConfig, ) -> Result<(), MissingVar>

Submits a single image to be rendered on the GPU, with extra variables

See submit for additional details.

pub fn map_image<'a>(&self, image: &'a mut ImageReadBuffer) -> MappedImage<'a>

Synchronously maps an image read buffer

The image read buffer should be populated by passing it as an argument when calling submit.

The image is borrowed exclusively to avoid double-mapping

This is a blocking function suitable for use on the desktop

pub async fn map_image_async<'a>( &self, image: &'a mut ImageReadBuffer, ) -> MappedImage<'a>

Asynchronously maps an image read buffer

The image read buffer should be populated by passing it as an argument when calling submit.

The image is borrowed exclusively to avoid double-mapping

This is an async function suitable for use in WebAssembly.

pub fn set_buffers_image_size( &self, buffers: &mut Buffers, image_size: VoxelSize, ) -> Result<(), BuffersError>

Resizes buffers to the given image size

Buffer allocations may grow but do not shrink; delete and recreate buffers if their capacity exceeds their size to a significant degree.