Strongly-typed buffers
This module is mostly internal to the crate, but is public because its types appear as return values and arguments.
Structs
- Generic
Flex Buffer - Handle around a growable GPU buffer
- Mapped
Buffer Tag - Helper
structto make a mapped version of a storage buffer - Mapped
Image - Handle to a mapped
ImageReadBuffer, which unmaps the image when dropped
Enums
- Buffer
Size Error - Error type when resizing a buffer beyond its limit
- Buffer
Type - Buffer type for error reporting
Traits
- Buffer
Item Count - Trait for types which have a certain number of items
- Buffer
Tag - Tag associated with a particular
GenericFlexBuffer
Type Aliases
- Array
Buffer - Resizable array buffer
- Image
Buffer - Resizable image buffer
- Image
Read Buffer - Buffer for reading data back from the GPU