pub enum BufferSizeError {
TooLarge {
requested_size: u64,
buffer_type: BufferType,
},
}Expand description
Error type when resizing a buffer beyond its limit
We check against maximum buffer sizes (from the WebGPU spec) and return an error immediately, instead of deferring the error to the point where the buffer is used.
Variants
TooLarge
Buffer size is too large for the requested buffer usage
Fields

buffer_type: BufferTypeBuffer type (which determines the max size)