pub struct Context { /* private fields */ }Expand description
WGPU context for applying various effects
Implementations
Sourceimpl Context
impl Context
Sourcepub fn merge_buffers(
&self,
image_size: VoxelSize,
) -> Result<MergeBuffers, BufferSizeError>
pub fn merge_buffers( &self, image_size: VoxelSize, ) -> Result<MergeBuffers, BufferSizeError>
Builds a new set of MergeBuffers for the given image size
Sourcepub fn shade_buffers(
&self,
image_size: ImageSize,
) -> Result<ShadeBuffers, BufferSizeError>
pub fn shade_buffers( &self, image_size: ImageSize, ) -> Result<ShadeBuffers, BufferSizeError>
Builds a new set of ShadeBuffers for the given image size
Sourcepub fn ssao_buffers(
&self,
image_size: VoxelSize,
) -> Result<SsaoBuffers, BufferSizeError>
pub fn ssao_buffers( &self, image_size: VoxelSize, ) -> Result<SsaoBuffers, BufferSizeError>
Builds a new set of SsaoBuffers for the given image size
Sourcepub fn submit_merge(
&self,
images: &[&ImageBuffer<GeomBufferTag>],
denoise: bool,
buf: &mut MergeBuffers,
) -> Result<(), MergeError>
pub fn submit_merge( &self, images: &[&ImageBuffer<GeomBufferTag>], denoise: bool, buf: &mut MergeBuffers, ) -> Result<(), MergeError>
Submits a set of merge operations to combine all of the images
The output buffer is resized to fit the images
If the incoming slice is empty, then no work is submitted
Sourcepub fn submit_shade(
&self,
image: &MergeBuffers,
ssao: Option<&SsaoBuffers>,
buf: &mut ShadeBuffers,
out: Option<&mut ImageReadBuffer<ShadedImageTag>>,
) -> Result<(), ShadeError>
pub fn submit_shade( &self, image: &MergeBuffers, ssao: Option<&SsaoBuffers>, buf: &mut ShadeBuffers, out: Option<&mut ImageReadBuffer<ShadedImageTag>>, ) -> Result<(), ShadeError>
Submits an operation to shade an image
The output buffer is resized to fit the images
Sourcepub fn submit_ssao(
&self,
image: &MergeBuffers,
buf: &mut SsaoBuffers,
) -> Result<(), SsaoError>
pub fn submit_ssao( &self, image: &MergeBuffers, buf: &mut SsaoBuffers, ) -> Result<(), SsaoError>
Submits a pass to compute an SSAO buffer