pub struct Gpu {
pub device: Device,
pub queue: Queue,
}Expand description
Handle to a GPU device
Fields
device: DeviceGPU device
queue: QueueGPU queue
Implementations
Sourceimpl Gpu
impl Gpu
Sourcepub async fn init() -> Result<Gpu, InitError>
pub async fn init() -> Result<Gpu, InitError>
Returns a Gpu object with customized settings
Non-default settings are as follows:
- We request a
wgpu::PowerPreference::HighPerformanceadapter - We enable the
wgpu::Features::TIMESTAMP_QUERYfeature
This is a helper function for simplicity; more sophisticated systems will likely construct the adapter, device, and queue themselves.
Sourcepub fn read_buffer_for<T: BufferTag>(
&self,
buf: &ImageBuffer<T>,
) -> ImageReadBuffer<T>
pub fn read_buffer_for<T: BufferTag>( &self, buf: &ImageBuffer<T>, ) -> ImageReadBuffer<T>
Returns a readable buffer for the given image buffer
Sourcepub fn map<'a, T: BufferTag>(
&self,
buf: &'a mut ImageReadBuffer<T>,
) -> MappedImage<'a, T>
pub fn map<'a, T: BufferTag>( &self, buf: &'a mut ImageReadBuffer<T>, ) -> MappedImage<'a, T>
Maps a readable image buffer, returning a mapped image